/*-
 * ~~~~~~licensing~~~~~~
 * schema-soap
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.registry.schema.soap.mapper;

import org.mapstruct.Mapping;
import org.mapstruct.Mappings;
import ru.entaxy.esb.system.registry.schema.api.entity.Resource;
import ru.entaxy.esb.system.registry.schema.api.entity.ResourceInfo;
import ru.entaxy.esb.system.registry.schema.soap.cxf.*;

@org.mapstruct.Mapper
public interface ResourceMapper {

    @Mappings({
            @Mapping(target = "createdDate", expression = "java(new java.util.Date())"),
            @Mapping(target = "resourceId", source = "idResource"),
    })
    ResourceInfo toResourceInfo(ResourceInfoType resourceInfoType);

    @Mappings({
            @Mapping(source = "resourceId", target = "idResource"),
    })
    ResourceInfoFullType toResourceInfoFullType(ResourceInfo resourceInfo);

    @Mappings({
            @Mapping(target = "resourceId", source = "idResource"),
            @Mapping(target = "editedDate", expression = "java(new java.util.Date())"),
    })
    ResourceInfo editedResourceInfoFullTypeResourceInfo(EditedResourceInfoFullType resourceInfoFullType);

    @Mappings({
            @Mapping(target = "createdDate", expression = "java(new java.util.Date())"),
            @Mapping(target = "resourceValue", source = "resourceValue"),
    })
    Resource toResource(LoadResourceType request);

    @Mappings({
            @Mapping(target = "editedDate", expression = "java(new java.util.Date())"),
    })
    Resource toEditedResource(ResourceType request);

    @Mappings({
            @Mapping(source = "resourceValue", target = "resourceValue"),
    })
    FullResourceType toFullResourceType(Resource resource);
}
